@@ -1,5 +1,5 @@
-use std::iter::Iterator;
 use super::{Regex, Region, SearchOptions};
+use std::iter::Iterator;
 
 impl Regex {
     /// Returns the capture groups corresponding to the leftmost-first match
@@ -13,7 +13,8 @@ impl Regex {
             text.len(),
             SearchOptions::SEARCH_OPTION_NONE,
             Some(&mut region),
-        ).map(|pos| Captures {
+        )
+        .map(|pos| Captures {
             text,
             region,
             offset: pos,
@@ -48,7 +49,7 @@ impl Regex {
             region: Region::new(),
             text,
             last_end: 0,
-            skip_next_empty: false,
+            last_match_end: None,
         }
     }
 
@@ -81,7 +82,7 @@ impl Regex {
             regex: self,
             text,
             last_end: 0,
-            skip_next_empty: false,
+            last_match_end: None,
         }
     }
 
@@ -159,12 +160,17 @@ impl Regex {
         let start = to_search.as_ptr();
         let end = to_search[to_search.len()..].as_ptr();
 
-        extern "C" fn scan_cb<F>(i: c_int, j: c_int, r: *const OnigRegion, ud: *mut c_void) -> c_int
+        unsafe extern "C" fn scan_cb<F>(
+            i: c_int,
+            j: c_int,
+            r: *mut OnigRegion,
+            ud: *mut c_void,
+        ) -> c_int
         where
             F: Fn(i32, i32, &Region) -> bool,
         {
             let region = Region::clone_from_raw(r);
-            let callback = unsafe { &*(ud as *mut F) };
+            let callback = &*(ud as *mut F);
             if callback(i, j, &region) {
                 0
             } else {
@@ -179,7 +185,7 @@ impl Regex {
                 end,
                 (&mut region.raw) as *mut ::onig_sys::OnigRegion,
                 options.bits(),
-                scan_cb::<F>,
+                Some(scan_cb::<F>),
                 &mut callback as *mut F as *mut c_void,
             )
         }
@@ -336,7 +342,7 @@ pub struct FindMatches<'r, 't> {
     region: Region,
     text: &'t str,
     last_end: usize,
-    skip_next_empty: bool,
+    last_match_end: Option<usize>,
 }
 
 impl<'r, 't> Iterator for FindMatches<'r, 't> {
@@ -355,22 +361,19 @@ impl<'r, 't> Iterator for FindMatches<'r, 't> {
             Some(&mut self.region),
         )?;
         let (s, e) = self.region.pos(0).unwrap();
-        self.last_end = e;
 
-        // Don't accept empty matches immediately following a match.
+        // Don't accept empty matches immediately following the last match.
         // i.e., no infinite loops please.
-        if e == s {
+        if e == s && self.last_match_end.map_or(false, |l| l == e) {
             self.last_end += self.text[self.last_end..]
                 .chars()
                 .next()
                 .map(|c| c.len_utf8())
                 .unwrap_or(1);
-            if self.skip_next_empty {
-                self.skip_next_empty = false;
-                return self.next();
-            }
+            return self.next();
         } else {
-            self.skip_next_empty = true;
+            self.last_end = e;
+            self.last_match_end = Some(e);
         }
 
         Some((s, e))
@@ -388,7 +391,7 @@ pub struct FindCaptures<'r, 't> {
     regex: &'r Regex,
     text: &'t str,
     last_end: usize,
-    skip_next_empty: bool,
+    last_match_end: Option<usize>,
 }
 
 impl<'r, 't> Iterator for FindCaptures<'r, 't> {
@@ -409,21 +412,18 @@ impl<'r, 't> Iterator for FindCaptures<'r, 't> {
         )?;
         let (s, e) = region.pos(0).unwrap();
 
-        // Don't accept empty matches immediately following a match.
+        // Don't accept empty matches immediately following the last match.
         // i.e., no infinite loops please.
-        if e == s {
+        if e == s && self.last_match_end.map_or(false, |l| l == e) {
             self.last_end += self.text[self.last_end..]
                 .chars()
                 .next()
                 .map(|c| c.len_utf8())
                 .unwrap_or(1);
-            if self.skip_next_empty {
-                self.skip_next_empty = false;
-                return self.next();
-            }
+            return self.next();
         } else {
             self.last_end = e;
-            self.skip_next_empty = true;
+            self.last_match_end = Some(e);
         }
         Some(Captures {
             text: self.text,
@@ -563,6 +563,13 @@ mod tests {
         assert_eq!(ms, vec![(0, 0), (1, 2), (3, 3), (4, 4), (5, 6)]);
     }
 
+    #[test]
+    fn test_find_iter_empty_after_match() {
+        let re = Regex::new(r"b|(?=,)").unwrap();
+        let ms = re.find_iter("ba,").collect::<Vec<_>>();
+        assert_eq!(ms, vec![(0, 1), (2, 2)]);
+    }
+
     #[test]
     fn test_zero_length_matches_jumps_past_match_location() {
         let re = Regex::new(r"\b").unwrap();
@@ -583,7 +590,8 @@ mod tests {
         let reg = Regex::new(r"\d+\.(\d+)").unwrap();
         let captures = reg.captures("100 - 3.1415 / 2.0").unwrap();
         assert_eq!(6, captures.offset());
-        let all_caps = reg.captures_iter("1 - 3234.3 * 123.2 - 100")
+        let all_caps = reg
+            .captures_iter("1 - 3234.3 * 123.2 - 100")
             .map(|cap| cap.offset())
             .collect::<Vec<_>>();
         assert_eq!(vec![4, 13], all_caps);
